"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createImageEditMenuProvider = void 0;
var createContextMenuProvider_1 = require("../utils/createContextMenuProvider");
var roosterjs_content_model_api_1 = require("roosterjs-content-model-api");
var roosterjs_content_model_dom_1 = require("roosterjs-content-model-dom");
var showInputDialog_1 = require("../../inputDialog/utils/showInputDialog");
var MIN_WIDTH = 10;
var MIN_HEIGHT = 10;
var ImageAltTextMenuItem = {
    key: 'menuNameImageAltText',
    unlocalizedText: 'Add alternate text',
    onClick: function (_, editor, node, strings, uiUtilities) {
        var image = node;
        var initValue = image.alt;
        (0, showInputDialog_1.showInputDialog)(uiUtilities, 'menuNameImageAltText', 'Add alternate text', {
            altText: {
                labelKey: null,
                unlocalizedLabel: null,
                initValue: initValue,
            },
        }, strings).then(function (values) {
            editor.focus();
            editor.setDOMSelection({
                type: 'image',
                image: image,
            });
            if (values) {
                (0, roosterjs_content_model_api_1.setImageAltText)(editor, values.altText);
            }
        });
    },
};
var sizeMap = {
    menuNameImageSizeBestFit: 0,
    menuNameImageSizeSmall: 0.25,
    menuNameImageSizeMedium: 0.5,
    menuNameImageSizeOriginal: 1,
};
var ImageResizeMenuItem = {
    key: 'menuNameImageResize',
    unlocalizedText: 'Size',
    subItems: {
        menuNameImageSizeBestFit: 'Best fit',
        menuNameImageSizeSmall: 'Small',
        menuNameImageSizeMedium: 'Medium',
        menuNameImageSizeOriginal: 'Original',
    },
    onClick: function (key, editor, _) {
        var _a;
        var selection = editor.getDOMSelection();
        if (!selection || selection.type !== 'image') {
            return;
        }
        var percentage = (_a = sizeMap[key]) !== null && _a !== void 0 ? _a : 0;
        if (percentage > 0) {
            var _b = selection.image, naturalWidth = _b.naturalWidth, naturalHeight = _b.naturalHeight;
            resizeByPercentage(editor, percentage, naturalWidth, naturalHeight);
        }
        else {
            resetImage(editor);
        }
    },
};
var ImageRotateMenuItem = {
    key: 'menuNameImageRotate',
    unlocalizedText: 'Rotate image',
    subItems: {
        menuNameImageRotateLeft: 'Left',
        menuNameImageRotateRight: 'Right',
    },
    shouldShow: function (_, node, imageEditor) {
        return (!!(imageEditor === null || imageEditor === void 0 ? void 0 : imageEditor.isOperationAllowed('rotate')) &&
            imageEditor.canRegenerateImage(node));
    },
    onClick: function (key, _editor, _node, _strings, _uiUtilities, imageEdit) {
        switch (key) {
            case 'menuNameImageRotateLeft':
                imageEdit === null || imageEdit === void 0 ? void 0 : imageEdit.rotateImage(-Math.PI / 2);
                break;
            case 'menuNameImageRotateRight':
                imageEdit === null || imageEdit === void 0 ? void 0 : imageEdit.rotateImage(Math.PI / 2);
                break;
        }
    },
};
var ImageFlipMenuItem = {
    key: 'menuNameImageFlip',
    unlocalizedText: 'Flip image',
    subItems: {
        menuNameImageRotateFlipHorizontally: 'Flip Horizontally',
        menuNameImageRotateFlipVertically: 'Flip Vertically',
    },
    shouldShow: function (_, node, imageEditor) {
        return (!!(imageEditor === null || imageEditor === void 0 ? void 0 : imageEditor.isOperationAllowed('rotate')) &&
            imageEditor.canRegenerateImage(node));
    },
    onClick: function (key, _editor, _node, _strings, _uiUtilities, imageEdit) {
        switch (key) {
            case 'menuNameImageRotateFlipHorizontally':
                imageEdit === null || imageEdit === void 0 ? void 0 : imageEdit.flipImage('horizontal');
                break;
            case 'menuNameImageRotateFlipVertically':
                imageEdit === null || imageEdit === void 0 ? void 0 : imageEdit.flipImage('vertical');
                break;
        }
    },
};
var ImageCropMenuItem = {
    key: 'menuNameImageCrop',
    unlocalizedText: 'Crop image',
    shouldShow: function (_, node, imageEditor) {
        return (!!(imageEditor === null || imageEditor === void 0 ? void 0 : imageEditor.isOperationAllowed('crop')) &&
            imageEditor.canRegenerateImage(node));
    },
    onClick: function (_, _editor, _node, _strings, _uiUtilities, imageEdit) {
        imageEdit === null || imageEdit === void 0 ? void 0 : imageEdit.cropImage();
    },
};
var ImageRemoveMenuItem = {
    key: 'menuNameImageRemove',
    unlocalizedText: 'Remove image',
    onClick: function (_, editor, node) {
        removeImage(editor);
    },
};
var ImageCopyMenuItem = {
    key: 'menuNameImageCopy',
    unlocalizedText: 'Copy image',
    onClick: function (_, editor) {
        var _a;
        (_a = editor.getDocument()) === null || _a === void 0 ? void 0 : _a.execCommand('copy');
    },
};
var ImageCutMenuItem = {
    key: 'menuNameImageCut',
    unlocalizedText: 'Cut image',
    onClick: function (_, editor) {
        var _a;
        (_a = editor.getDocument()) === null || _a === void 0 ? void 0 : _a.execCommand('cut');
    },
};
function shouldShowImageEditItems(editor, _) {
    var selection = editor.getDOMSelection();
    return (selection === null || selection === void 0 ? void 0 : selection.type) === 'image' && !!selection.image;
}
/**
 * Create a new instance of ContextMenuProvider to support image editing functionalities in context menu
 * @returns A new ContextMenuProvider
 */
function createImageEditMenuProvider(imageEditor, strings) {
    return (0, createContextMenuProvider_1.createContextMenuProvider)('imageEdit', [
        ImageAltTextMenuItem,
        ImageResizeMenuItem,
        ImageCropMenuItem,
        ImageRemoveMenuItem,
        ImageRotateMenuItem,
        ImageFlipMenuItem,
        ImageCopyMenuItem,
        ImageCutMenuItem,
    ], strings, shouldShowImageEditItems, imageEditor);
}
exports.createImageEditMenuProvider = createImageEditMenuProvider;
function removeImage(editor) {
    editor.formatContentModel(function (model) {
        var changed = false;
        (0, roosterjs_content_model_dom_1.iterateSelections)(model, function (_, __, block, segments) {
            segments === null || segments === void 0 ? void 0 : segments.forEach(function (segment) {
                if (segment.segmentType == 'Image' && (block === null || block === void 0 ? void 0 : block.blockType) == 'Paragraph') {
                    var index = block.segments.indexOf(segment);
                    if (index >= 0) {
                        (0, roosterjs_content_model_dom_1.mutateBlock)(block).segments.splice(index, 1);
                        changed = true;
                    }
                }
            });
        });
        return changed;
    }, {
        apiName: 'DeleteImage',
    });
}
function resizeByPercentage(editor, percentage, naturalWidth, naturalHeight) {
    (0, roosterjs_content_model_api_1.formatImageWithContentModel)(editor, 'resizeImage', function (segment) {
        (0, roosterjs_content_model_dom_1.updateImageMetadata)(segment, function (format) {
            var validFormat = Object.assign({
                naturalWidth: naturalWidth,
                naturalHeight: naturalHeight,
                leftPercent: 0,
                rightPercent: 0,
                topPercent: 0,
                bottomPercent: 0,
                angleRad: 0,
            }, format);
            var newWidth = Math.max(MIN_WIDTH, validFormat.naturalWidth *
                (1 - validFormat.leftPercent - validFormat.rightPercent) *
                percentage);
            var newHeight = Math.max(MIN_HEIGHT, validFormat.naturalHeight *
                (1 - validFormat.topPercent - validFormat.bottomPercent) *
                percentage);
            validFormat.widthPx = newWidth;
            validFormat.heightPx = newHeight;
            segment.format.width = newWidth + 'px';
            segment.format.height = newHeight + 'px';
            return validFormat;
        });
    });
}
function resetImage(editor) {
    (0, roosterjs_content_model_api_1.formatImageWithContentModel)(editor, 'resizeImage', function (segment) {
        (0, roosterjs_content_model_dom_1.updateImageMetadata)(segment, function () { return null; });
        delete segment.format.width;
        delete segment.format.height;
        segment.format.maxWidth = '100%';
    });
}
//# sourceMappingURL=createImageEditMenuProvider.js.map