"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createEmojiPlugin = void 0;
var tslib_1 = require("tslib");
var React = require("react");
var Utilities_1 = require("@fluentui/react/lib/Utilities");
var emojiList_1 = require("../utils/emojiList");
var showEmojiCallout_1 = require("../components/showEmojiCallout");
var roosterjs_content_model_core_1 = require("roosterjs-content-model-core");
var roosterjs_content_model_dom_1 = require("roosterjs-content-model-dom");
var EmojiStrings_1 = require("../type/EmojiStrings");
var KEYCODE_COLON = 186;
var KEYCODE_COLON_FIREFOX = 59;
// Regex looks for an emoji right before the : to allow contextual search immediately following an emoji
// MATCHES: 0: 😃:r
//          1: 😃
//          2: :r
var EMOJI_BEFORE_COLON_REGEX = /([\u0023-\u0039][\u20e3]|[\ud800-\udbff][\udc00-\udfff]|[\u00a9-\u00ae]|[\u2122-\u3299])*([:;][^:]*)/;
// White space matching regex. It matches following chars:
// \s: white space
// \u00A0: no-breaking white space
// \u200B: zero width space
// \u3000: full width space (which can come from JPN IME)
var WHITESPACE_REGEX = /[\s\u00A0\u200B\u3000]+([^\s\u00A0\u200B\u3000]*)$/i;
var EmojiPlugin = /** @class */ (function () {
    function EmojiPlugin(searchBoxStrings) {
        var _this = this;
        this.searchBoxStrings = searchBoxStrings;
        this.editor = null;
        this.eventHandledOnKeyDown = false;
        this.canUndoEmoji = false;
        this.isSuggesting = false;
        this.paneRef = React.createRef();
        this.timer = null;
        this.uiUtilities = null;
        this.emojiCalloutRef = React.createRef();
        this.baseId = 0;
        this.onHideCallout = function () { return _this.setIsSuggesting(false); };
        this.onSelectFromPane = function (emoji, wordBeforeCursor) {
            var _a;
            if (emoji === emojiList_1.MoreEmoji) {
                (_a = _this.paneRef.current) === null || _a === void 0 ? void 0 : _a.showFullPicker(wordBeforeCursor);
                return;
            }
            _this.insertEmoji(emoji, wordBeforeCursor);
        };
        this.strings = (0, tslib_1.__assign)((0, tslib_1.__assign)((0, tslib_1.__assign)({}, EmojiStrings_1.EmojiDescriptionStrings), EmojiStrings_1.EmojiKeywordStrings), EmojiStrings_1.EmojiFamilyStrings);
    }
    EmojiPlugin.prototype.setUIUtilities = function (uiUtilities) {
        this.uiUtilities = uiUtilities;
    };
    EmojiPlugin.prototype.getName = function () {
        return 'Emoji';
    };
    EmojiPlugin.prototype.dispose = function () {
        var _a;
        this.setIsSuggesting(false);
        (_a = this.emojiCalloutRef.current) === null || _a === void 0 ? void 0 : _a.dismiss();
        this.editor = null;
        this.baseId = 0;
    };
    EmojiPlugin.prototype.initialize = function (editor) {
        this.editor = editor;
    };
    EmojiPlugin.prototype.onPluginEvent = function (event) {
        if (!this.editor) {
            return;
        }
        if (event.eventType === 'keyDown') {
            this.eventHandledOnKeyDown = false;
            if (this.isSuggesting) {
                this.onKeyDownSuggestingDomEvent(this.editor, event);
            }
            else if (event.rawEvent.key === 'Backspace' && this.canUndoEmoji) {
                //TODO: 1051
                // If KeyDown is backspace and canUndoEmoji, call editor undo
                (0, roosterjs_content_model_core_1.undo)(this.editor);
                this.handleEventOnKeyDown(event);
                this.canUndoEmoji = false;
            }
        }
        else if (event.eventType === 'keyUp' && !(0, roosterjs_content_model_dom_1.isModifierKey)(event.rawEvent)) {
            if (this.isSuggesting) {
                this.onKeyUpSuggestingDomEvent(this.editor, event);
            }
            else {
                this.onKeyUpDomEvent(this.editor, event);
            }
        }
        else if (event.eventType === 'mouseUp') {
            //TODO: 1052
            // If MouseUp, the emoji cannot be undone
            this.canUndoEmoji = false;
            this.setIsSuggesting(false);
        }
    };
    /**
     * On KeyDown suggesting DOM event
     * Try to insert emoji is possible
     * Intercept arrow keys to move selection if popup is shown
     */
    EmojiPlugin.prototype.onKeyDownSuggestingDomEvent = function (editor, event) {
        // If key is enter, try insert emoji at selection
        // If key is space and selection is shortcut, try insert emoji
        var _a, _b;
        var wordBeforeCursor = this.getWordBeforeCursor(editor);
        switch (event.rawEvent.key) {
            case 'Enter':
                var selectedEmoji = (_a = this.paneRef.current) === null || _a === void 0 ? void 0 : _a.getSelectedEmoji();
                // check if selection is on the "..." and show full picker if so, otherwise try to apply emoji
                if (!selectedEmoji ||
                    !wordBeforeCursor ||
                    this.tryShowFullPicker(event, selectedEmoji, wordBeforeCursor)) {
                    break;
                }
                else {
                    this.insertEmoji(selectedEmoji, wordBeforeCursor);
                    this.handleEventOnKeyDown(event);
                }
                break;
            case 'ArrowLeft':
            case 'ArrowRight':
                (_b = this.paneRef.current) === null || _b === void 0 ? void 0 : _b.navigate(event.rawEvent.key === 'ArrowLeft' ? -1 : 1);
                this.handleEventOnKeyDown(event);
                break;
            case 'Escape':
                this.setIsSuggesting(false);
                this.handleEventOnKeyDown(event);
        }
    };
    EmojiPlugin.prototype.tryShowFullPicker = function (event, selectedEmoji, wordBeforeCursor) {
        var _a;
        if (selectedEmoji !== emojiList_1.MoreEmoji) {
            return false;
        }
        this.handleEventOnKeyDown(event);
        (_a = this.paneRef.current) === null || _a === void 0 ? void 0 : _a.showFullPicker(wordBeforeCursor);
        return true;
    };
    /**
     * On KeyUp suggesting DOM event
     * If key is character, update search term
     * Otherwise set isSuggesting to false
     */
    EmojiPlugin.prototype.onKeyUpSuggestingDomEvent = function (editor, event) {
        var _a, _b, _c, _d;
        if (this.eventHandledOnKeyDown) {
            return;
        }
        // If this is a character key or backspace
        // Clear the timer as we will either queue a new timer or stop suggesting
        if (this.timer &&
            ((event.rawEvent.key.length === 1 && event.rawEvent.which !== Utilities_1.KeyCodes.space) ||
                event.rawEvent.which === Utilities_1.KeyCodes.backspace)) {
            (_b = (_a = this.editor) === null || _a === void 0 ? void 0 : _a.getDocument().defaultView) === null || _b === void 0 ? void 0 : _b.clearTimeout(this.timer);
            this.timer = null;
            (_c = this.emojiCalloutRef.current) === null || _c === void 0 ? void 0 : _c.dismiss();
        }
        var wordBeforeCursor = this.getWordBeforeCursor(editor);
        if (wordBeforeCursor) {
            if (this.paneRef) {
                (_d = this.paneRef.current) === null || _d === void 0 ? void 0 : _d.setSearch(wordBeforeCursor);
            }
            else {
                this.setIsSuggesting(false);
            }
        }
        else {
            this.setIsSuggesting(false);
        }
    };
    EmojiPlugin.prototype.onKeyUpDomEvent = function (editor, event) {
        if (this.eventHandledOnKeyDown) {
            return;
        }
        if ((event.rawEvent.which === KEYCODE_COLON ||
            event.rawEvent.which === KEYCODE_COLON_FIREFOX) &&
            this.getWordBeforeCursor(editor) === ':') {
            this.setIsSuggesting(true);
        }
    };
    EmojiPlugin.prototype.getCallout = function () {
        var _a;
        var selection = (_a = this.editor) === null || _a === void 0 ? void 0 : _a.getDOMSelection();
        var rangeNode = (selection === null || selection === void 0 ? void 0 : selection.type) == 'range' ? selection.range.startContainer : null;
        var rangeElement = (0, roosterjs_content_model_dom_1.isNodeOfType)(rangeNode, 'ELEMENT_NODE')
            ? rangeNode
            : rangeNode === null || rangeNode === void 0 ? void 0 : rangeNode.parentElement;
        var rect = rangeElement === null || rangeElement === void 0 ? void 0 : rangeElement.getBoundingClientRect();
        if (this.uiUtilities && rect) {
            this.baseId++;
            (0, showEmojiCallout_1.showEmojiCallout)(this.uiUtilities, rect, this.strings, this.onSelectFromPane, this.paneRef, this.emojiCalloutRef, this.onHideCallout, this.baseId, this.searchBoxStrings);
        }
    };
    EmojiPlugin.prototype.setIsSuggesting = function (isSuggesting) {
        var _a;
        if (this.isSuggesting === isSuggesting) {
            return;
        }
        this.isSuggesting = isSuggesting;
        if (this.isSuggesting) {
            this.getCallout();
        }
        else if (this.emojiCalloutRef) {
            (_a = this.emojiCalloutRef.current) === null || _a === void 0 ? void 0 : _a.dismiss();
        }
    };
    EmojiPlugin.prototype.insertEmoji = function (emoji, wordBeforeCursor) {
        var _a;
        if (!wordBeforeCursor || !this.editor || !emoji.codePoint) {
            return;
        }
        this.editor.formatContentModel(function (model) {
            (0, roosterjs_content_model_dom_1.iterateSelections)(model, function (_, __, block, segments) {
                if ((block === null || block === void 0 ? void 0 : block.blockType) == 'Paragraph' &&
                    (segments === null || segments === void 0 ? void 0 : segments.length) == 1 &&
                    segments[0].segmentType == 'SelectionMarker') {
                    var index = block.segments.indexOf(segments[0]);
                    var previousSegment = block.segments[index - 1];
                    if ((previousSegment === null || previousSegment === void 0 ? void 0 : previousSegment.segmentType) == 'Text' &&
                        previousSegment.text.endsWith(wordBeforeCursor)) {
                        (0, roosterjs_content_model_dom_1.mutateSegment)(block, previousSegment, function (segment) {
                            segment.text =
                                segment.text.substring(0, segment.text.length - wordBeforeCursor.length) + emoji.codePoint;
                        });
                    }
                }
                return true;
            });
            return true;
        });
        (_a = this.emojiCalloutRef.current) === null || _a === void 0 ? void 0 : _a.dismiss();
    };
    EmojiPlugin.prototype.getWordBeforeCursor = function (editor) {
        var wordBeforeCursor = null;
        editor.formatContentModel(function (model) {
            (0, roosterjs_content_model_dom_1.iterateSelections)(model, function (_, __, block, segments) {
                if ((block === null || block === void 0 ? void 0 : block.blockType) == 'Paragraph' &&
                    (segments === null || segments === void 0 ? void 0 : segments.length) == 1 &&
                    segments[0].segmentType == 'SelectionMarker') {
                    var index = block.segments.indexOf(segments[0]);
                    var prevSegment = block.segments[index - 1];
                    if ((prevSegment === null || prevSegment === void 0 ? void 0 : prevSegment.segmentType) == 'Text') {
                        // Match on the white space, the portion after space is on the index of 1 of the matched result
                        // (index at 0 is whole match result, index at 1 is the word)
                        var matches_1 = WHITESPACE_REGEX.exec(prevSegment.text);
                        wordBeforeCursor = (matches_1 === null || matches_1 === void 0 ? void 0 : matches_1.length) == 2 ? matches_1[1] : prevSegment.text;
                    }
                }
                return true;
            });
            return false;
        });
        var matches = wordBeforeCursor ? EMOJI_BEFORE_COLON_REGEX.exec(wordBeforeCursor) : null;
        return matches && matches.length > 2 && matches[0] === wordBeforeCursor ? matches[2] : null;
    };
    EmojiPlugin.prototype.handleEventOnKeyDown = function (event) {
        this.eventHandledOnKeyDown = true;
        event.rawEvent.preventDefault();
        event.rawEvent.stopImmediatePropagation();
    };
    return EmojiPlugin;
}());
/**
 * Create a new instance of Emoji plugin with FluentUI components.
 */
function createEmojiPlugin(searchBoxStrings) {
    return new EmojiPlugin(searchBoxStrings);
}
exports.createEmojiPlugin = createEmojiPlugin;
//# sourceMappingURL=createEmojiPlugin.js.map